# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO
from enum import Enum


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Wfm1000z(KaitaiStruct):
    """Rigol DS1000Z scope .wmf file format.
    """

    class UnitEnum(Enum):
        w = 0
        a = 1
        v = 2
        u = 3

    class AcquistionEnum(Enum):
        normal = 0
        peak = 1
        average = 2
        high_resolution = 3

    class TimeModeEnum(Enum):
        yt = 0
        xy = 1
        roll = 2

    class ProbeEnum(Enum):
        x0_01 = 0
        x0_02 = 1
        x0_05 = 2
        x0_1 = 3
        x0_2 = 4
        x0_5 = 5
        x1 = 6
        x2 = 7
        x5 = 8
        x10 = 9
        x20 = 10
        x50 = 11
        x100 = 12
        x200 = 13
        x500 = 14
        x1000 = 15

    class BandwidthEnum(Enum):
        mhz_20 = 0
        no_limit = 1

    class CouplingEnum(Enum):
        dc = 0
        ac = 1
        gnd = 2
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        pass

    class ChannelSubhead(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.unknown_1 = self._io.read_bytes(3)
            self.unused_bits_1 = self._io.read_bits_int_be(7)
            self.enabled = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.unknown_2 = self._io.read_bytes(7)
            self.unused_bits_2 = self._io.read_bits_int_be(7)
            self.inverted = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.unknown_3 = self._io.read_bytes(10)
            self.probe_attenuation = self._io.read_s8le()
            self.unknown_4 = self._io.read_bytes(16)
            self.label = (KaitaiStream.bytes_terminate(self._io.read_bytes(4), 0, False)).decode(u"UTF-8")
            self.unknown_5 = self._io.read_bytes(10)


    class RawData(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.raw = self._io.read_bytes((self._root.header.points * self._root.header.stride))


    class ChannelHead(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.enabled_val = self._io.read_u1()
            self.coupling = KaitaiStream.resolve_enum(Wfm1000z.CouplingEnum, self._io.read_u1())
            self.bandwidth_limit = KaitaiStream.resolve_enum(Wfm1000z.BandwidthEnum, self._io.read_u1())
            self.probe_type = self._io.read_u1()
            self.probe_ratio = KaitaiStream.resolve_enum(Wfm1000z.ProbeEnum, self._io.read_u1())
            self.unused = self._io.read_bytes(3)
            self.scale = self._io.read_f4le()
            self.shift = self._io.read_f4le()
            self.inverted_val = self._io.read_u1()
            self.unit = KaitaiStream.resolve_enum(Wfm1000z.UnitEnum, self._io.read_u1())
            self.unknown_2 = self._io.read_bytes(10)

        @property
        def y_scale(self):
            if hasattr(self, '_m_y_scale'):
                return self._m_y_scale

            self._m_y_scale = (-(self.volt_per_division) / 20.0)
            return getattr(self, '_m_y_scale', None)

        @property
        def y_offset(self):
            if hasattr(self, '_m_y_offset'):
                return self._m_y_offset

            self._m_y_offset = (self.shift - self.volt_per_division)
            return getattr(self, '_m_y_offset', None)

        @property
        def time_offset(self):
            if hasattr(self, '_m_time_offset'):
                return self._m_time_offset

            self._m_time_offset = self._root.header.time_offset
            return getattr(self, '_m_time_offset', None)

        @property
        def inverted(self):
            if hasattr(self, '_m_inverted'):
                return self._m_inverted

            self._m_inverted = (True if self.inverted_val != 0 else False)
            return getattr(self, '_m_inverted', None)

        @property
        def time_scale(self):
            if hasattr(self, '_m_time_scale'):
                return self._m_time_scale

            self._m_time_scale = self._root.header.time_scale
            return getattr(self, '_m_time_scale', None)

        @property
        def volt_offset(self):
            if hasattr(self, '_m_volt_offset'):
                return self._m_volt_offset

            self._m_volt_offset = self.shift
            return getattr(self, '_m_volt_offset', None)

        @property
        def volt_per_division(self):
            if hasattr(self, '_m_volt_per_division'):
                return self._m_volt_per_division

            self._m_volt_per_division = ((-1.0 * self.scale) if self.inverted else (1.0 * self.scale))
            return getattr(self, '_m_volt_per_division', None)

        @property
        def volt_scale(self):
            if hasattr(self, '_m_volt_scale'):
                return self._m_volt_scale

            self._m_volt_scale = (self.volt_per_division / 25.0)
            return getattr(self, '_m_volt_scale', None)

        @property
        def probe_value(self):
            if hasattr(self, '_m_probe_value'):
                return self._m_probe_value

            self._m_probe_value = (0.01 if self.probe_ratio == Wfm1000z.ProbeEnum.x0_01 else (0.02 if self.probe_ratio == Wfm1000z.ProbeEnum.x0_02 else (0.05 if self.probe_ratio == Wfm1000z.ProbeEnum.x0_05 else (0.1 if self.probe_ratio == Wfm1000z.ProbeEnum.x0_1 else (0.2 if self.probe_ratio == Wfm1000z.ProbeEnum.x0_2 else (0.5 if self.probe_ratio == Wfm1000z.ProbeEnum.x0_5 else (1.0 if self.probe_ratio == Wfm1000z.ProbeEnum.x1 else (2.0 if self.probe_ratio == Wfm1000z.ProbeEnum.x2 else (5.0 if self.probe_ratio == Wfm1000z.ProbeEnum.x5 else (10.0 if self.probe_ratio == Wfm1000z.ProbeEnum.x10 else (20.0 if self.probe_ratio == Wfm1000z.ProbeEnum.x20 else (50.0 if self.probe_ratio == Wfm1000z.ProbeEnum.x50 else (100.0 if self.probe_ratio == Wfm1000z.ProbeEnum.x100 else (200.0 if self.probe_ratio == Wfm1000z.ProbeEnum.x200 else (500.0 if self.probe_ratio == Wfm1000z.ProbeEnum.x500 else 1000.0)))))))))))))))
            return getattr(self, '_m_probe_value', None)

        @property
        def enabled(self):
            if hasattr(self, '_m_enabled'):
                return self._m_enabled

            self._m_enabled = (True if self.enabled_val != 0 else False)
            return getattr(self, '_m_enabled', None)


    class FileHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.magic = self._io.read_bytes(4)
            if not self.magic == b"\x01\xFF\xFF\xFF":
                raise kaitaistruct.ValidationNotEqualError(b"\x01\xFF\xFF\xFF", self.magic, self._io, u"/types/file_header/seq/0")
            self.magic2 = self._io.read_u2le()
            self.structure_size = self._io.read_u2le()
            self.model_number = (KaitaiStream.bytes_terminate(self._io.read_bytes(20), 0, False)).decode(u"ascii")
            self.firmware_version = (KaitaiStream.bytes_terminate(self._io.read_bytes(20), 0, False)).decode(u"ascii")
            self.block = self._io.read_bytes(2)
            if not self.block == b"\x01\x00":
                raise kaitaistruct.ValidationNotEqualError(b"\x01\x00", self.block, self._io, u"/types/file_header/seq/5")
            self.file_version = self._io.read_u2le()


    class WfmHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.picoseconds_per_division = self._io.read_u8le()
            self.picoseconds_offset = self._io.read_s8le()
            self.crc = self._io.read_u4le()
            self.structure_size = self._io.read_bytes(2)
            if not self.structure_size == b"\xD8\x00":
                raise kaitaistruct.ValidationNotEqualError(b"\xD8\x00", self.structure_size, self._io, u"/types/wfm_header/seq/3")
            self.structure_version = self._io.read_u2le()
            self.unused_bits_1 = self._io.read_bits_int_be(4)
            self.ch4_enabled = self._io.read_bits_int_be(1) != 0
            self.ch3_enabled = self._io.read_bits_int_be(1) != 0
            self.ch2_enabled = self._io.read_bits_int_be(1) != 0
            self.ch1_enabled = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.unused_mask_bytes = self._io.read_bytes(3)
            self.ch1_file_offset = self._io.read_u4le()
            self.ch2_file_offset = self._io.read_u4le()
            self.ch3_file_offset = self._io.read_u4le()
            self.ch4_file_offset = self._io.read_u4le()
            self.la_offset = self._io.read_u4le()
            self.acq_mode = KaitaiStream.resolve_enum(Wfm1000z.AcquistionEnum, self._io.read_u1())
            self.average_time = self._io.read_u1()
            self.sample_mode = self._io.read_bytes(1)
            if not self.sample_mode == b"\x00":
                raise kaitaistruct.ValidationNotEqualError(b"\x00", self.sample_mode, self._io, u"/types/wfm_header/seq/18")
            self.time_mode = KaitaiStream.resolve_enum(Wfm1000z.TimeModeEnum, self._io.read_u1())
            self.memory_depth = self._io.read_u4le()
            self.sample_rate_ghz = self._io.read_f4le()
            self.ch = []
            for i in range(4):
                self.ch.append(Wfm1000z.ChannelHead(self._io, self, self._root))

            self.la_parameters = self._io.read_bytes(12)
            self.setup_size = self._io.read_u4le()
            self.setup_offset = self._io.read_u4le()
            self.horizontal_size = self._io.read_u4le()
            self.horizontal_offset = self._io.read_u4le()

        @property
        def points(self):
            if hasattr(self, '_m_points'):
                return self._m_points

            self._m_points = self.memory_depth // self.stride
            return getattr(self, '_m_points', None)

        @property
        def ch2_int(self):
            if hasattr(self, '_m_ch2_int'):
                return self._m_ch2_int

            self._m_ch2_int = (1 if self.ch2_enabled else 0)
            return getattr(self, '_m_ch2_int', None)

        @property
        def sample_rate_hz(self):
            if hasattr(self, '_m_sample_rate_hz'):
                return self._m_sample_rate_hz

            self._m_sample_rate_hz = (self.sample_rate_ghz * 1E+9)
            return getattr(self, '_m_sample_rate_hz', None)

        @property
        def time_offset(self):
            if hasattr(self, '_m_time_offset'):
                return self._m_time_offset

            self._m_time_offset = (self.picoseconds_offset * 1E-12)
            return getattr(self, '_m_time_offset', None)

        @property
        def ch4_int(self):
            if hasattr(self, '_m_ch4_int'):
                return self._m_ch4_int

            self._m_ch4_int = (1 if self.ch4_enabled else 0)
            return getattr(self, '_m_ch4_int', None)

        @property
        def ch3_int(self):
            if hasattr(self, '_m_ch3_int'):
                return self._m_ch3_int

            self._m_ch3_int = (1 if self.ch3_enabled else 0)
            return getattr(self, '_m_ch3_int', None)

        @property
        def total_channels(self):
            if hasattr(self, '_m_total_channels'):
                return self._m_total_channels

            self._m_total_channels = (1 if self.total_count == 0 else self.total_count)
            return getattr(self, '_m_total_channels', None)

        @property
        def time_scale(self):
            if hasattr(self, '_m_time_scale'):
                return self._m_time_scale

            self._m_time_scale = (self.picoseconds_per_division * 1E-12)
            return getattr(self, '_m_time_scale', None)

        @property
        def seconds_per_point(self):
            if hasattr(self, '_m_seconds_per_point'):
                return self._m_seconds_per_point

            self._m_seconds_per_point = (1 / self.sample_rate_hz)
            return getattr(self, '_m_seconds_per_point', None)

        @property
        def ch1_int(self):
            if hasattr(self, '_m_ch1_int'):
                return self._m_ch1_int

            self._m_ch1_int = (1 if self.ch1_enabled else 0)
            return getattr(self, '_m_ch1_int', None)

        @property
        def stride(self):
            if hasattr(self, '_m_stride'):
                return self._m_stride

            self._m_stride = (4 if self.total_channels == 3 else self.total_channels)
            return getattr(self, '_m_stride', None)

        @property
        def total_count(self):
            if hasattr(self, '_m_total_count'):
                return self._m_total_count

            self._m_total_count = (((self.ch1_int + self.ch2_int) + self.ch3_int) + self.ch4_int)
            return getattr(self, '_m_total_count', None)


    @property
    def preheader(self):
        if hasattr(self, '_m_preheader'):
            return self._m_preheader

        _pos = self._io.pos()
        self._io.seek(0)
        self._m_preheader = Wfm1000z.FileHeader(self._io, self, self._root)
        self._io.seek(_pos)
        return getattr(self, '_m_preheader', None)

    @property
    def header(self):
        if hasattr(self, '_m_header'):
            return self._m_header

        _pos = self._io.pos()
        self._io.seek(64)
        self._m_header = Wfm1000z.WfmHeader(self._io, self, self._root)
        self._io.seek(_pos)
        return getattr(self, '_m_header', None)

    @property
    def data(self):
        if hasattr(self, '_m_data'):
            return self._m_data

        _pos = self._io.pos()
        self._io.seek(((304 + self._root.header.setup_size) + self._root.header.horizontal_size))
        self._m_data = Wfm1000z.RawData(self._io, self, self._root)
        self._io.seek(_pos)
        return getattr(self, '_m_data', None)





# pylint: disable=invalid-name
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=consider-using-f-string

"""
Class structure and methods for an oscilloscope channel.

The idea is to collect all the relevant information from all the Rigol
scope waveforms into a single structure that can be handled in a uniform
and consistent manner.

Specifically this lets one do::

    channel.times   : numpy array of signal times
    channel.volts   : numpy array of signal voltages

or the stringification method to describe a channel::

    print(channel)

"""
from enum import Enum
import numpy as np


class UnitEnum(Enum):
    """Enumerated units for scopes without them."""

    w = 0
    a = 1
    v = 2
    u = 3


def best_scale(number):
    """Scale and units for a number with proper prefix."""
    absnr = abs(number)

    if absnr == 0:
        return 1, ' '
    if absnr < 0.99999999e-9:
        return 1e12, 'p'
    if absnr < 0.99999999e-6:
        return 1e9, 'n'
    if absnr < 0.99999999e-3:
        return 1e6, 'µ'
    if absnr < 0.99999999:
        return 1e3, 'm'
    if absnr < 0.99999999e3:
        return 1, ' '
    if absnr < 0.99999999e6:
        return 1e-3, 'k'
    if absnr < 0.999999991e9:
        return 1e-6, 'M'
    return 1e-9, 'G'


def engineering_string(number, n_digits):
    """Format number with proper prefix."""
    scale, prefix = best_scale(number)
    fformat = "%%.%df %%s" % n_digits
    s = fformat % (number * scale, prefix)
    return s


def _channel_bytes(channel_number, w):
    """
    Return right series of bytes for a channel for 1000Z scopes.

    Waveform points are interleaved stored in memory when two or more
    channels are saved.  This unweaves them.

    Args:
        channel_number: the number of enabled channels before this one
        w:              original waveform object
    Returns
        byte array for specified channel
    """
    offset = 0

    if w.header.stride == 2:   # byte pattern e.g., CH4 CH1 CH4 CH 1
        # use odd bytes when this is the second enabled channel
        if not any(w.header.ch[i].enabled for i in range(channel_number - 1)):
            offset = 1

    elif w.header.stride == 4:  # byte pattern CH4 CH3 CH2 CH1
        offset = 4 - channel_number

    data = np.frombuffer(w.data.raw, dtype=np.uint8)
    raw_bytes = data[offset::w.header.stride]

    return raw_bytes


class Channel():
    """Base class for a single channel."""

    def __init__(self, w, channel_number, scope, selected='1234'):
        """
        Initialize a Channel Object.

        Args:
            w: Wfm object
            channel_number: 1, 2, 3, or 4
            scope: string describing scope
            selected: string with channels chosen by user
        Returns:
            Channel object
        """
        self.channel_number = channel_number
        self.name = f'CH {channel_number}'
        self.waveform = w
        self.seconds_per_point = w.header.seconds_per_point
        self.firmware = 'unknown'
        self.unit = UnitEnum.v
        self.points = 0
        self.raw = None
        self.volts = None
        self.times = None
        self.coupling = 'unknown'
        self.roll_stop = 0
        self.time_offset = 0
        self.time_scale = 1
        self.enabled = False
        self.enabled_and_selected = False
        self.volt_scale = 1
        self.volt_offset = 0
        self.y_scale = 1
        self.y_offset = 0
        self.volt_per_division = 1
        self.probe_value = 1
        self.inverted = False

        # determine if this channel is one of those chosen by user
        chosen = str(channel_number) in selected

        if channel_number <= len(w.header.ch):
            channel = w.header.ch[channel_number - 1]
            self.enabled = channel.enabled
            self.enabled_and_selected = channel.enabled and chosen
            self.volt_scale = channel.volt_scale
            self.volt_offset = channel.volt_offset
            self.y_scale = channel.volt_scale
            self.y_offset = channel.volt_offset
            self.volt_per_division = channel.volt_per_division
            self.probe_value = channel.probe_value
            self.unit = channel.unit
            self.inverted = channel.inverted

        if scope == 'wfm1000b':
            self.y_offset += 1.12 * channel.volt_per_division
            self.ds1000b(w, channel_number)
        elif scope == 'wfm1000c':
            self.ds1000c(w, channel_number)
        elif scope == 'wfm1000d':
            self.ds1000d(w, channel_number)
        elif scope == 'wfm1000e':
            self.ds1000e(w, channel_number)
        elif scope == 'wfm1000z':
            self.ds1000z(w, channel_number)
        elif scope == 'wfm2000':
            self.ds2000(w, channel_number)
        elif scope == 'wfm4000':
            self.ds4000(w, channel_number)
        elif scope == 'wfm6000':
            self.ds6000(w, channel_number)

    def __str__(self):
        """Describe this channel."""
        s = "     Channel %d:\n" % self.channel_number
        s += "         Coupling = %8s\n" % self.coupling.rjust(7, ' ')
        s += "            Scale = %10sV/div\n" % engineering_string(self.volt_per_division, 2)
        s += "           Offset = %10sV\n" % engineering_string(self.volt_offset, 2)
        s += "            Probe = %7gX\n" % self.probe_value
        s += "         Inverted = %8s\n\n" % self.inverted
        s += "        Time Base = %10ss/div\n" % engineering_string(self.time_scale, 3)
        s += "           Offset = %10ss\n" % engineering_string(self.time_offset, 3)
        s += "            Delta = %10ss/point\n" % engineering_string(self.seconds_per_point, 3)
        s += "           Points = %8d\n\n" % self.points

        if self.enabled_and_selected:
            format_str = "         Count    = [%9d,%9d,%9d  ... %9d,%9d]\n"
            s += format_str % (1, 2, 3, self.points - 1, self.points)
            format_str = "           Raw    = [%9d,%9d,%9d  ... %9d,%9d]\n"
            s += format_str % (self.raw[0], self.raw[1], self.raw[2], self.raw[-2], self.raw[-1])
            t = [engineering_string(self.times[i], 3) + "s" for i in [0, 1, 2, -2, -1]]
            format_str = "           Times  = [%9s,%9s,%9s  ... %9s,%9s]\n"
            s += format_str % (t[0], t[1], t[2], t[-2], t[-1])
            v = [engineering_string(self.volts[i], 2) + "V" for i in [0, 1, 2, -2, -1]]
            format_str = "           Volts  = [%9s,%9s,%9s  ... %9s,%9s]\n"
            s += format_str % (v[0], v[1], v[2], v[-2], v[-1])
        return s

    def calc_times_and_volts(self):
        """Calculate the times and voltages for this channel."""
        if self.enabled_and_selected:
            self.volts = self.y_scale * (127.0 - self.raw) - self.y_offset
            h = self.points * self.seconds_per_point / 2
            self.times = np.linspace(-h, h, self.points) + self.time_offset

    def ds1000b(self, w, channel_number):
        """Interpret waveform data for 1000B series scopes."""
        self.time_scale = 1.0e-12 * w.header.time_scale
        self.time_offset = 1.0e-12 * w.header.time_offset
        self.coupling = 'AC'
        if channel_number == 1:
            if self.enabled_and_selected:
                if (w.header.coupling_ch12 & 0xC0) == 0xC0:
                    self.coupling = 'DC'
                self.points = w.header.len_ch1
                self.raw = np.frombuffer(w.header.ch1, dtype=np.uint8)

        if channel_number == 2:
            if self.enabled_and_selected:
                if (w.header.coupling_ch12 & 0x0C) == 0x0C:
                    self.coupling = 'DC'
                self.points = w.header.len_ch2
                self.raw = np.frombuffer(w.header.ch2, dtype=np.uint8)

        if channel_number == 3:
            if self.enabled_and_selected:
                if (w.header.coupling_ch34 & 0xC0) == 0xC0:
                    self.coupling = 'DC'
                self.points = w.header.len_ch3
                self.raw = np.frombuffer(w.header.ch3, dtype=np.uint8)

        if channel_number == 4:
            if self.enabled_and_selected:
                if (w.header.coupling_ch34 & 0x0C) == 0x0C:
                    self.coupling = 'DC'
                self.points = w.header.len_ch4
                self.raw = np.frombuffer(w.header.ch4, dtype=np.uint8)

        self.calc_times_and_volts()

    def ds1000c(self, w, channel_number):
        """Interpret waveform data for 1000CD series scopes."""
        self.time_scale = 1.0e-12 * w.header.time_scale
        self.time_offset = 1.0e-12 * w.header.time_offset
        if channel_number == 1:
            if self.enabled_and_selected:
                self.points = len(w.data.ch1)
                self.raw = np.frombuffer(w.data.ch1, dtype=np.uint8)

        if channel_number == 2:
            if self.enabled_and_selected:
                self.points = len(w.data.ch2)
                self.raw = np.frombuffer(w.data.ch2, dtype=np.uint8)

        self.calc_times_and_volts()

    def ds1000d(self, w, channel_number):
        """Interpret waveform data for 1000CD series scopes."""
        self.time_scale = 1.0e-12 * w.header.time_scale
        self.time_offset = 1.0e-12 * w.header.time_offset
        if channel_number == 1:
            if self.enabled_and_selected:
                self.points = len(w.data.ch1)
                self.raw = np.frombuffer(w.data.ch1, dtype=np.uint8)

        if channel_number == 2:
            if self.enabled_and_selected:
                self.points = len(w.data.ch2)
                self.raw = np.frombuffer(w.data.ch2, dtype=np.uint8)

        self.calc_times_and_volts()

    def ds1000e(self, w, channel_number):
        """Interpret waveform data for 1000D and 1000E series scopes."""
        self.roll_stop = w.header.roll_stop

        if channel_number == 1:
            self.time_offset = w.header.ch1_time_offset
            self.time_scale = w.header.ch1_time_scale
            if self.enabled_and_selected:
                self.points = len(w.data.ch1)
                self.raw = np.frombuffer(w.data.ch1, dtype=np.uint8)

        elif channel_number == 2:
            self.time_offset = w.header.ch2_time_offset
            self.time_scale = w.header.ch2_time_scale
            if self.enabled_and_selected:
                self.points = len(w.data.ch2)
                self.raw = np.frombuffer(w.data.ch2, dtype=np.uint8)

        self.calc_times_and_volts()

    def ds1000z(self, w, channel_number):
        """Interpret waveform for the Rigol DS1000Z series."""
        self.time_scale = w.header.time_scale
        self.time_offset = w.header.time_offset
        self.points = w.header.points
        self.stride = w.header.stride
        self.firmware = w.preheader.firmware_version
        self.probe = w.header.ch[channel_number - 1].probe_value
        self.coupling = w.header.ch[channel_number - 1].coupling.name.upper()
        self.y_scale = w.header.ch[channel_number - 1].y_scale
        self.y_offset = w.header.ch[channel_number - 1].y_offset

        if self.enabled_and_selected:
            self.raw = _channel_bytes(channel_number, w)
            self.points = len(self.raw)

        self.calc_times_and_volts()

    def ds2000(self, w, channel_number):
        """Interpret waveform for the Rigol DS2000 series."""
        self.time_offset = w.header.time_offset
        self.time_scale = w.header.time_scale
        self.points = w.header.storage_depth
        self.firmware = w.header.firmware_version
        self.unit = UnitEnum(w.header.ch[channel_number - 1].unit_actual)
        self.coupling = w.header.ch[channel_number - 1].coupling.name.upper()
        self.y_scale = -self.volt_scale
        self.y_offset = self.volt_offset

        if self.enabled_and_selected and not w.header.enabled.interwoven:
            if channel_number == 1:
                self.raw = np.frombuffer(w.header.raw_1, dtype=np.uint8)

            if channel_number == 2:
                self.raw = np.frombuffer(w.header.raw_2, dtype=np.uint8)

            if channel_number == 3:
                self.raw = np.frombuffer(w.header.raw_3, dtype=np.uint8)

            if channel_number == 4:
                self.raw = np.frombuffer(w.header.raw_4, dtype=np.uint8)

        elif w.header.enabled.interwoven:
            # 'Interwoven' wave captures use the memory available to all channels
            # to sample at a higher resolution.  This means if CH1 is disabled
            # CH2 will use the memory from CH1.
            self.raw = np.empty((self.points,), dtype=np.uint8)
            self.raw[0::2] = np.frombuffer(w.header.raw_1, count=self.points // 2, dtype=np.uint8)
            self.raw[1::2] = np.frombuffer(w.header.raw_2, count=self.points // 2, dtype=np.uint8)

        self.calc_times_and_volts()

    def ds4000(self, w, channel_number):
        """Interpret waveform for the Rigol DS4000 series."""
        self.time_offset = w.header.time_offset
        self.time_scale = w.header.time_scale
        self.points = w.header.points
        self.firmware = w.header.firmware_version
        self.coupling = w.header.ch[channel_number - 1].coupling.name.upper()
        self.y_scale = -self.volt_scale
        self.y_offset = self.volt_offset

        if self.enabled_and_selected:
            if channel_number == 1:
                self.raw = np.frombuffer(w.header.raw_1, dtype=np.uint8)

            if channel_number == 2:
                self.raw = np.frombuffer(w.header.raw_2, dtype=np.uint8)

            if channel_number == 3:
                self.raw = np.frombuffer(w.header.raw_3, dtype=np.uint8)

            if channel_number == 4:
                self.raw = np.frombuffer(w.header.raw_4, dtype=np.uint8)

        self.calc_times_and_volts()

    def ds6000(self, w, channel_number):
        """Interpret waveform for the Rigol DS6000 series."""
        self.time_offset = w.header.time_offset
        self.time_scale = w.header.time_scale
        self.points = w.header.points
        self.firmware = w.header.firmware_version
        self.coupling = w.header.ch[channel_number - 1].coupling.name.upper()
        self.unit = w.header.ch[channel_number - 1].unit

        if self.enabled_and_selected:
            if channel_number == 1:
                self.raw = np.array(w.header.raw_1, dtype=np.uint8)

            if channel_number == 2:
                self.raw = np.array(w.header.raw_2, dtype=np.uint8)

            if channel_number == 3:
                self.raw = np.array(w.header.raw_3, dtype=np.uint8)

            if channel_number == 4:
                self.raw = np.array(w.header.raw_4, dtype=np.uint8)

        self.calc_times_and_volts()



# pylint: disable=invalid-name
# pylint: disable=too-many-instance-attributes
# pylint: disable=raise-missing-from
# pylint: disable=too-many-branches
# pylint: disable=too-many-statements
# pylint: disable=consider-using-f-string
# pylint: disable=consider-using-with

"""
Extract signals or description from Rigol 1000E Oscilloscope waveform file.

Example:
    >>> import RigolWFM.wfm as rigol
    >>> waveform = rigol.Wfm.from_file("file_name.wfm", 'E')
    >>> description = waveform.describe()
    >>> print(description)

"""
import os.path
import tempfile
import urllib.parse
import wave

import matplotlib.pyplot as plt
import numpy as np
import requests

import RigolWFM.wfm1000b
import RigolWFM.wfm1000c
import RigolWFM.wfm1000d
import RigolWFM.wfm1000e
import RigolWFM.wfm1000z
import RigolWFM.wfm2000
import RigolWFM.wfm4000
import RigolWFM.wfm6000
import RigolWFM.channel

# in progress
DS1000B_scopes = ["B", "1000B", "DS1000B",
                  "DS1074B", "DS1104B", "DS1204B"]

# tested
DS1000C_scopes = ["C", "1000C", "DS1000C",
                  "DS1000CD", "DS1000C", "DS1000MD", "DS1000M",
                  "DS1302CA", "DS1202CA", "DS1102CA", "DS1062CA",
                  "DS1042C"]

# tested
DS1000D_scopes = ["D", "1000D", "DS1000D", "DS1102D", "DS1052D"]

# tested
DS1000E_scopes = ["E", "1000E", "DS1000E", "DS1102E", "DS1052E"]

# tested, wonky voltages
DS1000Z_scopes = ["Z", "1000Z", "DS1000Z",
                  "DS1202Z",
                  "DS1054Z", "MSO1054Z",
                  "DS1074Z", "MSO1074Z", "DS1074Z-S",
                  "DS1104Z", "MSO1104Z", "DS1104Z-S"]

# tested
DS2000_scopes = ["2", "2000", "DS2000", "DS2072A",
                 "DS2102A", "MSO2102A", "MSO2102A-S",
                 "DS2202A", "MSO2202A", "MSO2202A-S",
                 "DS2302A", "MSO2302A", "MSO2302A-S"]

# tested
DS4000_scopes = ["4", "4000", "DS4000",
                 "DS4054", "DS4052", "DS4034", "DS4032", "DS4024",
                 "DS4022", "DS4014", "DS4012", "MSO4054", "MSO4052", "MSO4034",
                 "MSO4032", "MSO4024", "MSO4022", "MSO4014", "MSO4012"]

# untested
DS6000_scopes = ["6", "6000", "DS6000",
                 "DS6062", "DS6064", "DS6102", "DS6104"]


def valid_scope_list():
    """List all the oscilloscope types."""
    s = "\nRigol oscilloscope models:\n    "
    s += ", ".join(DS1000C_scopes) + "\n    "
    s += ", ".join(DS1000D_scopes) + "\n    "
    s += ", ".join(DS1000E_scopes) + "\n    "
    s += ", ".join(DS1000Z_scopes) + "\n    "
    s += ", ".join(DS2000_scopes) + "\n    "
    s += ", ".join(DS4000_scopes) + "\n    "
    s += ", ".join(DS6000_scopes) + "\n"
    return s


class Read_WFM_Error(Exception):
    """Generic Read Error."""


class Parse_WFM_Error(Exception):
    """Generic Parse Error."""


class Invalid_URL(Exception):
    """Cannot use this URL.  It must start with https or http."""


class Unknown_Scope_Error(Exception):
    """Not one of the listed Rigol oscilloscopes."""


class Write_WAV_Error(Exception):
    """Something went wrong while writing the .wave file."""


class Channel_Not_In_WFM_Error(Exception):
    """The channel is not in the .wfm file."""


class Wfm():
    """Class with parsed data from a .wfm file."""

    def __init__(self, file_name):
        """Initialize a Wfm object from a file."""
        self.channels = []
        self.original_name = file_name
        self.file_name = file_name
        self.basename = file_name
        self.firmware = 'unknown'

        # there are multiple possible scope names
        # 1. user_name   - the name passed to the program
        # 2. header_name - the name found in the header of the file
        # 3. parser_name - the name of parser used
        self.user_name = 'unknown'
        self.parser_name = 'unknown'
        self.header_name = 'unknown'

    @classmethod
    def from_file(cls, file_name, model, selected='1234'):
        """
        Create Wfm object from a file.

        Args:
            file_name: name of file
            model: Rigol Oscilloscope used, e.g., 'E' or 'Z'
            selected: string of channels to process e.g., '12'
        Returns:
            a wfm object for the file
        """
        # ensure that file exists
        try:
            f = open(file_name, 'rb')
            f.close()
        except IOError as e:
            raise Read_WFM_Error(e)

        new_wfm = cls(file_name)
        new_wfm.original_name = file_name
        new_wfm.file_name = file_name
        new_wfm.basename = os.path.basename(file_name)

        # parse the waveform
        umodel = model.upper()

        if umodel in DS1000B_scopes:
            w = RigolWFM.wfm1000b.Wfm1000b.from_file(file_name)
            new_wfm.header_name = 'DS1000B'

        elif umodel in DS1000C_scopes:
            w = RigolWFM.wfm1000c.Wfm1000c.from_file(file_name)
            new_wfm.header_name = 'DS1000C'

        elif umodel in DS1000D_scopes:
            w = RigolWFM.wfm1000d.Wfm1000d.from_file(file_name)
            new_wfm.header_name = 'DS1000D'

        elif umodel in DS1000E_scopes:
            w = RigolWFM.wfm1000e.Wfm1000e.from_file(file_name)
            new_wfm.header_name = 'DS1000E'

        elif umodel in DS1000Z_scopes:
            w = RigolWFM.wfm1000z.Wfm1000z.from_file(file_name)
            new_wfm.header_name = w.preheader.model_number

        elif umodel in DS2000_scopes:
            w = RigolWFM.wfm2000.Wfm2000.from_file(file_name)
            new_wfm.header_name = 'DS2000'

        elif umodel in DS4000_scopes:
            w = RigolWFM.wfm4000.Wfm4000.from_file(file_name)
            new_wfm.header_name = w.header.model_number

        elif umodel in DS6000_scopes:
            w = RigolWFM.wfm6000.Wfm6000.from_file(file_name)
            new_wfm.header_name = w.header.model_number

        else:
            print("Unknown Rigol oscilloscope type: '%s'" % umodel)
            print(valid_scope_list())
            return new_wfm

        new_wfm.user_name = model
        pname = str(w).split(".")[1]
        new_wfm.parser_name = pname

        # assemble into uniform set of names
        enabled = ''
        for ch_number in range(1, 5):

            ch = RigolWFM.channel.Channel(w, ch_number, pname, selected)

            if not ch.enabled:
                continue

            # keep track of enabled channels for error message
            enabled += str(ch_number)

            # append all enabled channels
            new_wfm.channels.append(ch)

        if len(new_wfm.channels) == 0:
            print("Sorry! No channels in the waveform are both selected and enabled")
            print("    User selected channels = '%s'" % selected)
            print("    Scope enabled channels = '%s'" % enabled)
            print()
        else:
            new_wfm.firmware = new_wfm.channels[0].firmware

        return new_wfm

    @classmethod
    def from_url(cls, url, model, selected='1234'):
        """
        Return a waveform object given a URL.

        This is a bit complicated because the parser must have a local file
        to work with.  The process is to download the file to a temporary
        location and then process that file.  There is a lot that can go
        wrong - bad url, bad download, or an error parsing the file.

        Args:
            url: location of the file
            model: Rigol Oscilloscope used, e.g., 'E' or 'Z'
            selected: string of channels to process e.g., '12'
        Returns:
            a wfm object for the file
        """
        u = urllib.parse.urlparse(url)
        scheme = u[0]

        if scheme not in ['http', 'https']:
            raise Invalid_URL()

        try:
            # need a local file for conversion, download url and save as tempfile
            print("downloading '%s'" % url)
            r = requests.get(url, allow_redirects=True, timeout=10)
            r.raise_for_status()

            if not r.ok:
                error_string = "Downloading URL '%s' failed: '%s'" % (
                    url, r.reason)
                raise Read_WFM_Error(error_string)

            f = tempfile.NamedTemporaryFile()
            f.write(r.content)
            f.seek(0)
            working_name = f.name

            try:
                new_wfm = cls.from_file(working_name, model, selected)
                new_wfm.original_name = url
                # extract the simple name
                rawpath = u[2]
                path = urllib.parse.unquote(rawpath)
                new_wfm.basename = os.path.basename(path)
                return new_wfm
            except Exception as e:
                raise Parse_WFM_Error(e)

        except r.exceptions.RequestException as e:
            raise Parse_WFM_Error(e)

    def describe(self):
        """Return a string describing the contents of a Rigol wfm file."""
        s = "    General:\n"
        s += '        File Model   = %s\n' % self.parser_name
        s += "        User Model   = %s\n" % self.user_name
        s += '        Parser Model = %s\n' % self.parser_name
        s += "        Firmware     = %s\n" % self.firmware
        s += '        Filename     = %s\n' % self.basename
        s += '        Channels     = ['

        first = True
        for ch in self.channels:
            if not first:
                s += ', '
            s += '%s' % ch.channel_number
            first = False
        s += ']\n\n'

        for ch in self.channels:
            s += str(ch)
            s += "\n"
        return s

    def best_scaling(self):
        """Return appropriate scaling for plot."""
        v_scale = 1e-12
        v_prefix = ''
        h_scale = 1e-12
        h_prefix = ''
        for ch in self.channels:
            v, p = RigolWFM.channel.best_scale(ch.volt_per_division)
            if v > v_scale:
                v_scale = v
                v_prefix = p
            h, p = RigolWFM.channel.best_scale(ch.time_scale)
            if h > h_scale:
                h_scale = h
                h_prefix = p
        return h_scale, h_prefix, v_scale, v_prefix

    def plot(self):
        """Plot the data."""
        h_scale, h_prefix, v_scale, v_prefix = self.best_scaling()
        colors = ['red', 'blue', 'orange', 'magenta']

        for i, ch in enumerate(self.channels):
            plt.plot(ch.times * h_scale, ch.volts * v_scale,
                     label=ch.name, color=colors[i])

        plt.xlabel("Time (%ss)" % h_prefix)
        plt.ylabel("Voltage (%sV)" % v_prefix)
        plt.title(self.basename)
        plt.legend(loc='upper right')

    def csv(self):
        """Return a string of comma separated values."""
        if len(self.channels) == 0:
            return ''

        h_scale, h_prefix, v_scale, v_prefix = self.best_scaling()

        s = 'X'
        for ch in self.channels:
            s += ",%s" % ch.name
        s += ",Start,Increment\n"

        # just output the display 100 pts/division
        ch = self.channels[0]
        incr = ch.time_scale / 100
        off = -6 * ch.time_scale
        s += '%ss' % h_prefix
        for ch in self.channels:
            s += ",%s%s" % (v_prefix, ch.unit.name.upper())
        s += ",%e,%e\n" % (off, incr)

        for i in range(self.channels[0].points):
            s += "%.6f" % (ch.times[i] * h_scale)
            for ch in self.channels:
                s += ",%.2f" % (ch.volts[i] * v_scale)
            s += "\n"
        return s

    def sigrokcsv(self):
        """Return a string of comma separated values for sigrok."""
        if len(self.channels) == 0:
            return ''

        s = 'X'
        for ch in self.channels:
            s += ",%s (%s)" % (ch.name, ch.unit.name.upper())
        s += "\n"

        ch = self.channels[0]
        for i in range(self.channels[0].points):
            s += "%.8f" % (ch.times[i])
            for ch in self.channels:
                s += ",%.2f" % (ch.volts[i])
            s += "\n"
        return s

    def wav(self, wav_filename, autoscale=False):
        """Save data as a WAV file for use with LTspice or Sigrok."""
        n_channels = len(self.channels)
        channel_length = self.channels[0].points
        total_len = channel_length * n_channels

        out = np.empty((total_len,), dtype=np.uint8, order='C')

        # channels are interleaved e.g., 123123123
        for i, ch in enumerate(self.channels):
            if autoscale:
                amin = np.min(ch.raw)
                amax = max(np.max(ch.raw), amin + 1)  # avoid division by zero
                scale = 250 / (amax - amin) * 0.95
                out[i::n_channels] = np.int8((ch.raw - amin) * scale)
            else:
                out[i::n_channels] = ch.raw

        sample_rate = 1.0 / self.channels[0].seconds_per_point

        wavef = wave.open(wav_filename, 'wb')
        wavef.setnchannels(n_channels)  # 1 = mono, 2 = stereo
        wavef.setsampwidth(1)
        wavef.setframerate(sample_rate)
        wavef.setcomptype('NONE', '')
        wavef.setnframes(channel_length)

        wavef.writeframes(out)
        wavef.close()



#!/usr/bin/env python3

# pylint: disable=invalid-name
# pylint: disable=missing-function-docstring
# pylint: disable=unused-argument
# pylint: disable=consider-using-f-string

"""
Command line utility to convert Rigol .wfm files.

    Examples::

        prompt> wfmconvert E info DS1102E-A.wfm

        prompt> wfmconvert E csv DS1102E-A.wfm

        prompt> wfmconvert E wav DS1102E-A.wfm
"""

import re
import os
import sys
import shutil
import argparse
import subprocess
import textwrap

import RigolWFM.wfm as rigol


def info(args, scope_data, infile):
    """Create a string that describes content of .wfm file."""
    s = scope_data.describe()
    print(s)


def csv(args, scope_data, infile):
    """Create a file with comma separated values."""
    csv_name = os.path.splitext(infile)[0] + '.csv'

    if os.path.isfile(csv_name) and not args.force:
        print("'%s' exists, use --force to overwrite" % csv_name)
        return

    s = scope_data.csv()
    with open(csv_name, 'wb') as f:
        b = s.encode(encoding='utf-8')
        f.write(b)


def vcsv(args, scope_data, infile):
    """Create a file with comma separated values (full volts)."""
    csv_name = os.path.splitext(infile)[0] + '.csv'

    if os.path.isfile(csv_name) and not args.force:
        print("'%s' exists, use --force to overwrite" % csv_name)
        return

    s = scope_data.sigrokcsv()
    with open(csv_name, 'wb') as f:
        b = s.encode(encoding='utf-8')
        f.write(b)


def wav(args, scope_data, infile):
    """Create an audible .wav file for use in LTspice."""
    wav_name = os.path.splitext(infile)[0] + '.wav'
    if os.path.isfile(wav_name) and not args.force:
        print("'%s' exists, use --force to overwrite" % wav_name)
        return

    scope_data.wav(wav_name, autoscale=args.autoscale)


def sigrok(args, scope_data, infile):
    """Create a Sigrok (.sr) file."""
    sigrok_name = os.path.splitext(infile)[0] + '.sr'

    if os.path.isfile(sigrok_name) and not args.force:
        print(f"'{sigrok_name}' exists, use --force to overwrite", file=sys.stderr)
        return False

    s = scope_data.sigrokcsv()

    # Check if sigrok-cli is installed and accessible
    if not shutil.which("sigrok-cli"):
        print("sigrok-cli is not installed or not found in PATH.", file=sys.stderr)
        print("See https://sigrok.org/wiki/Sigrok-cli for more information.", file=sys.stderr)
        return False

    # sigrok-cli reports a warning about /dev/stdin not being a regular file,
    # but the conversion works fine.
    try:
        p = subprocess.run(
            ['sigrok-cli', '-I', 'csv:start_line=2:column_formats=t,1a',
             '-i', '/dev/stdin', '-o', sigrok_name],
            input=s,
            check=True,
            stderr=subprocess.PIPE,
            text=True)
    except subprocess.CalledProcessError as e:
        print(f"sigrok-cli failed with error: {e.stderr}", file=sys.stderr)
        print("See https://sigrok.org/wiki/Sigrok-cli for more information.", file=sys.stderr)
        return False
    except Exception as e:
        print(f"An unexpected error occurred: {e}", file=sys.stderr)
        return False

    if p.returncode != 0:
        print(f"sigrok-cli returned non-zero exit code: {p.returncode}", file=sys.stderr)
        print("See https://sigrok.org/wiki/Sigrok-cli for more information.", file=sys.stderr)
        return False

    return True

def main():
    """Parse console command line arguments."""
    parser = argparse.ArgumentParser(
        prog='wfmconvert',
        description='Convert Rigol WFM files to another format.',
        formatter_class=argparse.RawTextHelpFormatter,
        epilog=textwrap.dedent("""\
        examples:
            wfmconvert E info DS1102E.wfm
            wfmconvert --channel 2 E csv DS1102E.wfm
            wfmconvert --channel 124 E vcsv DS1102E.wfm
            wfmconvert --channel 34 --autoscale E wav DS1102E.wfm
        """) + rigol.valid_scope_list()
    )

    parser.add_argument(
        '--force',
        action='store_true',
        default=False,
        help="overwrite existing output files"
    )

    parser.add_argument(
        '--autoscale',
        action='store_true',
        default=False,
        help=textwrap.dedent("""\
        autoscale each channel to full range.  Used when creating
        .wav files so signal goes from 0-255.
        """)
    )

    parser.add_argument(
        '--channel',
        type=str,
        default='1234',
        help=textwrap.dedent("""\
        select channel(s) to process.  `--channel 1` outputs only contents of
        the first channel.  `--channel 34` outputs contents of channels 3 and 4.
        The default is `--channel 1234`.
        """)
    )

    parser.add_argument(
        'model',
        type=str,
        choices=['B', 'C', 'D', 'E', 'Z', '2', '4', '6'],
        help='oscilloscope model.  See list below.'
    )

    parser.add_argument(
        dest='action',
        choices=['csv', 'info', 'wav', 'vcsv', 'sigrok'],
        help=textwrap.dedent("""\
        csv:    convert to a file with comma separated values
        info:   show the various scope settings for a .wfm file
        wav:    convert to a WAV sound format file for use with Audacity
                or Sigrok Pulseview. If a single channel is specified then
                the .wav file can be used with LTspice.
        vcsv:   convert to a file with comma separated values with raw voltages
        sigrok: convert to a sigrok file
        """)
    )

    parser.add_argument(
        'infile',
        type=str,
        nargs='+',
        help="the WFM file to be converted"
    )

    args = parser.parse_args()

    # strip anything that is not a possible channel number
    good = re.sub(r'[^1234]', '', args.channel)

    # remove duplicates keeping order
    selected = "".join(dict.fromkeys(good))

    if len(selected) == 0:
        print('\nwfmconvert error')
        print('No valid channels were passed after --channel')
        print('Channels are identified by number and must be a combination of 1, 2, 3, or 4')
        print('You used "--channel %s"' % args.channel)
        sys.exit()

    if len(selected) > 4:
        print('\nwfmconvert error')
        print('Only 1-2 channel (monophonic) or 3-4 channels (stereo) ', end='')
        print('can be used when creating a .wav file')
        print('You used "--channel %s"' % args.channel)
        sys.exit()

    actionMap = {"info": info, "csv": csv, "wav": wav, "vcsv": vcsv, "sigrok": sigrok}

    for filename in args.infile:
        try:
            scope_data = rigol.Wfm.from_file(filename, args.model, selected)
            actionMap[args.action](args, scope_data, filename)

        except rigol.Parse_WFM_Error as e:
            print("File contents do not follow the format for", end='', file=sys.stderr)
            print("for the Rigol Oscilloscope Model %s." % args.model, file=sys.stderr)
            print("To help with development, please report this error\n", file=sys.stderr)
            print("as an issue to https://github.com/scottprahl/RigolWFM\n", file=sys.stderr)
            print(e, file=sys.stderr)
            sys.exit()


if __name__ == "__main__":
    main()



import subprocess


def test_wfmconvert_info():
    commands = [
#        "wfmconvert B info tests/DS1204B-A.wfm",
#        "wfmconvert C info tests/DS1202CA-A.wfm",
        "wfmconvert D info tests/DS1102D-A.wfm",
#        "wfmconvert E info tests/DS1102E-A.wfm",
#        "wfmconvert Z info tests/MSO1104.wfm",
#        "wfmconvert 2 info tests/DS2202.wfm",
#        "wfmconvert 4 info tests/DS4022-A.wfm"
    ]

    for command in commands:
        result = subprocess.run(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
        assert result.returncode == 0

def test_wfmconvert_csv():
    commands = [
#        "wfmconvert --force B csv tests/DS1204B-A.wfm",
#        "wfmconvert --force C csv tests/DS1202CA-A.wfm",
        "wfmconvert --force D csv tests/DS1102D-A.wfm",
#        "wfmconvert --force E csv tests/DS1102E-A.wfm",
#        "wfmconvert --force Z csv tests/MSO1104.wfm",
#        "wfmconvert --force 2 csv tests/DS2202.wfm",
#        "wfmconvert --force 4 csv tests/DS4022-A.wfm"
    ]

    for command in commands:
        result = subprocess.run(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
        assert result.returncode == 0

def test_wfmconvert_wav():
    commands = [
#        "wfmconvert --force B wav tests/DS1204B-A.wfm",
#        "wfmconvert --force C wav tests/DS1202CA-A.wfm",
        "wfmconvert --force D wav tests/DS1102D-A.wfm",
#        "wfmconvert --force E wav tests/DS1102E-A.wfm",
#        "wfmconvert --force Z wav tests/MSO1104.wfm",
#        "wfmconvert --force 2 wav tests/DS2202.wfm",
#        "wfmconvert --force 4 wav tests/DS4022-A.wfm"
    ]

    for command in commands:
        result = subprocess.run(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
        assert result.returncode == 0

def test_wfmconvert_vcsv():
    commands = [
#        "wfmconvert --force B vcsv tests/DS1204B-A.wfm",
#        "wfmconvert --force C vcsv tests/DS1202CA-A.wfm",
        "wfmconvert --force D vcsv tests/DS1102D-A.wfm",
#        "wfmconvert --force E vcsv tests/DS1102E-A.wfm",
#        "wfmconvert --force Z vcsv tests/MSO1104.wfm",
#        "wfmconvert --force 2 vcsv tests/DS2202.wfm",
#        "wfmconvert --force 4 vcsv tests/DS4022-A.wfm"
    ]

    for command in commands:
        result = subprocess.run(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
        assert result.returncode == 0

# Run the tests
if __name__ == "__main__":
    test_wfmconvert_info()
    test_wfmconvert_csv()
    test_wfmconvert_wav()
    test_wfmconvert_vcsv()
    print("All tests passed!")




"""
A utility to process Rigol oscilloscope wfm files.

For full documentation see <https://RigolWFM.readthedocs.io>

    Example:
    >>> import matplotlib.pyplot as plt
    >>> import RigolWFM.wfm as rigol

    >>> filename = 'example.wfm'
    >>> scope = 'DS1000E'

    >>> w = rigol.Wfm.from_file(filename, scope)
    >>> w.plot()
    >>> plt.show()

Info waveform class::

    help(rigol.wfm)

Info about channel class::

    help(rigol.channel)

All the parsing code is auto-generated by `kaitai-struct-compiler`
based on `.ksy` files that describe the binary format of the Rigol
oscilloscope `.wfm` files.
"""
__version__ = '0.9.9'
__author__ = 'Scott Prahl'
__email__ = 'scott.prahl@oit.edu'
__copyright__ = '2020-23, Scott Prahl'
__license__ = 'BSD 3-clause'
__url__ = 'https://github.com/scottprahl/RigolWFM.git'



import subprocess

def test_wfmconvert_sigrok():
    commands = [
#        "wfmconvert --force B sigrok tests/DS1204B-A.wfm",
#        "wfmconvert --force C sigrok tests/DS1202CA-A.wfm",
        "wfmconvert --force D sigrok tests/DS1102D-A.wfm",
#        "wfmconvert --force E sigrok tests/DS1102E-A.wfm",
#        "wfmconvert --force Z sigrok tests/MSO1104.wfm",
#        "wfmconvert --force 2 sigrok tests/DS2202.wfm",
#        "wfmconvert --force 4 sigrok tests/DS4022-A.wfm"
    ]

    for command in commands:
        result = subprocess.run(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
        assert result.returncode == 0

# Run the tests
if __name__ == "__main__":
    test_wfmconvert_sigrok()
    print("All tests passed!")




# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO
from enum import Enum


if getattr(kaitaistruct, 'API_VERSION', (0, 9)) < (0, 9):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Wfm1000b(KaitaiStruct):
    """This was put together based on an excel header list of unknown provenance.
    It has been tested with a handful of different files.  The offset to the
    data seems correct and the channel coupling is untested.
    """

    class TriggerSourceEnum(Enum):
        ch1 = 0
        ch2 = 1
        ext = 2
        ext5 = 3
        ac_line = 5
        dig_ch = 7

    class TriggerModeEnum(Enum):
        edge = 0
        pulse = 1
        slope = 2
        video = 3
        alt = 4
        pattern = 5
        duration = 6

    class MachineModeEnum(Enum):
        ds1000b = 0
        ds1000c = 1
        ds1000e = 2
        ds1000z = 3
        ds2000 = 4
        ds4000 = 5
        ds6000 = 6

    class UnitEnum(Enum):
        w = 0
        a = 1
        v = 2
        u = 3
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        pass

    class Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.magic = self._io.read_bytes(4)
            if not self.magic == b"\xA5\xA5\xA4\x01":
                raise kaitaistruct.ValidationNotEqualError(b"\xA5\xA5\xA4\x01", self.magic, self._io, u"/types/header/seq/0")
            self.scopetype = (KaitaiStream.bytes_terminate(self._io.read_bytes(8), 0, False)).decode(u"UTF-8")
            self.unknown_1 = self._io.read_bytes(44)
            self.adcmode = self._io.read_u1()
            self.unknown_2 = self._io.read_bytes(3)
            self.points = self._io.read_u4le()
            self.active_channel = self._io.read_u1()
            self.unknown_3 = self._io.read_bytes(3)
            self.ch = []
            for i in range(4):
                self.ch.append(Wfm1000b.ChannelHeader(self._io, self, self._root))

            self.time_scale = self._io.read_u8le()
            self.time_offset = self._io.read_s8le()
            self.sample_rate_hz = self._io.read_f4le()
            self.time_scale_stop = self._io.read_u8le()
            self.time_scale_offset = self._io.read_s8le()
            self.unknown_4 = []
            for i in range(4):
                self.unknown_4.append(self._io.read_u4le())

            self.coupling_ch12 = self._io.read_u1()
            self.coupling_ch34 = self._io.read_u1()
            self.unknown_5 = self._io.read_bytes(4)
            self.trigger_mode = KaitaiStream.resolve_enum(Wfm1000b.TriggerModeEnum, self._io.read_u1())
            self.unknown_6 = self._io.read_u1()
            self.trigger_source = KaitaiStream.resolve_enum(Wfm1000b.TriggerSourceEnum, self._io.read_u1())

        @property
        def ch2(self):
            if hasattr(self, '_m_ch2'):
                return self._m_ch2

            if self.ch[1].enabled:
                io = self._root._io
                _pos = io.pos()
                io.seek((420 + self._root.header.points))
                self._m_ch2 = io.read_bytes(self.len_ch2)
                io.seek(_pos)

            return getattr(self, '_m_ch2', None)

        @property
        def len_ch1(self):
            if hasattr(self, '_m_len_ch1'):
                return self._m_len_ch1

            self._m_len_ch1 = (self.points if self.ch[0].enabled else 0)
            return getattr(self, '_m_len_ch1', None)

        @property
        def ch1(self):
            if hasattr(self, '_m_ch1'):
                return self._m_ch1

            if self.ch[0].enabled:
                io = self._root._io
                _pos = io.pos()
                io.seek(420)
                self._m_ch1 = io.read_bytes(self.len_ch1)
                io.seek(_pos)

            return getattr(self, '_m_ch1', None)

        @property
        def len_ch4(self):
            if hasattr(self, '_m_len_ch4'):
                return self._m_len_ch4

            self._m_len_ch4 = (self.points if self.ch[3].enabled else 0)
            return getattr(self, '_m_len_ch4', None)

        @property
        def len_ch3(self):
            if hasattr(self, '_m_len_ch3'):
                return self._m_len_ch3

            self._m_len_ch3 = (self.points if self.ch[2].enabled else 0)
            return getattr(self, '_m_len_ch3', None)

        @property
        def len_ch2(self):
            if hasattr(self, '_m_len_ch2'):
                return self._m_len_ch2

            self._m_len_ch2 = (self.points if self.ch[1].enabled else 0)
            return getattr(self, '_m_len_ch2', None)

        @property
        def ch4(self):
            if hasattr(self, '_m_ch4'):
                return self._m_ch4

            if self.ch[3].enabled:
                io = self._root._io
                _pos = io.pos()
                io.seek((420 + (self._root.header.points * 3)))
                self._m_ch4 = io.read_bytes(self.len_ch4)
                io.seek(_pos)

            return getattr(self, '_m_ch4', None)

        @property
        def seconds_per_point(self):
            if hasattr(self, '_m_seconds_per_point'):
                return self._m_seconds_per_point

            self._m_seconds_per_point = (1.0 / self.sample_rate_hz)
            return getattr(self, '_m_seconds_per_point', None)

        @property
        def ch3(self):
            if hasattr(self, '_m_ch3'):
                return self._m_ch3

            if self.ch[2].enabled:
                io = self._root._io
                _pos = io.pos()
                io.seek((420 + (self._root.header.points * 2)))
                self._m_ch3 = io.read_bytes(self.len_ch3)
                io.seek(_pos)

            return getattr(self, '_m_ch3', None)


    class ChannelHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.scale_display = self._io.read_s4le()
            self.shift_display = self._io.read_s2le()
            self.unknown1 = self._io.read_bytes(2)
            self.probe_value = self._io.read_f4le()
            self.probe_type = self._io.read_s1()
            self.invert_disp_val = self._io.read_u1()
            self.enabled_val = self._io.read_u1()
            self.invert_m_val = self._io.read_u1()
            self.scale_measured = self._io.read_s4le()
            self.shift_measured = self._io.read_s2le()
            self.time_delayed = self._io.read_u1()
            self.unknown2 = self._io.read_bytes(1)

        @property
        def unit(self):
            if hasattr(self, '_m_unit'):
                return self._m_unit

            self._m_unit = Wfm1000b.UnitEnum.v
            return getattr(self, '_m_unit', None)

        @property
        def time_offset(self):
            if hasattr(self, '_m_time_offset'):
                return self._m_time_offset

            self._m_time_offset = self._root.header.time_offset
            return getattr(self, '_m_time_offset', None)

        @property
        def inverted(self):
            if hasattr(self, '_m_inverted'):
                return self._m_inverted

            self._m_inverted = (True if self.invert_m_val != 0 else False)
            return getattr(self, '_m_inverted', None)

        @property
        def time_scale(self):
            if hasattr(self, '_m_time_scale'):
                return self._m_time_scale

            self._m_time_scale = self._root.header.time_scale
            return getattr(self, '_m_time_scale', None)

        @property
        def volt_offset(self):
            if hasattr(self, '_m_volt_offset'):
                return self._m_volt_offset

            self._m_volt_offset = (self.shift_measured * self.volt_scale)
            return getattr(self, '_m_volt_offset', None)

        @property
        def volt_per_division(self):
            if hasattr(self, '_m_volt_per_division'):
                return self._m_volt_per_division

            self._m_volt_per_division = (((-0.0000010 * self.scale_measured) * self.probe_value) if self.inverted else ((0.0000010 * self.scale_measured) * self.probe_value))
            return getattr(self, '_m_volt_per_division', None)

        @property
        def volt_scale(self):
            if hasattr(self, '_m_volt_scale'):
                return self._m_volt_scale

            self._m_volt_scale = (self.volt_per_division / 25.0)
            return getattr(self, '_m_volt_scale', None)

        @property
        def enabled(self):
            if hasattr(self, '_m_enabled'):
                return self._m_enabled

            self._m_enabled = (True if self.enabled_val != 0 else False)
            return getattr(self, '_m_enabled', None)


    @property
    def header(self):
        if hasattr(self, '_m_header'):
            return self._m_header

        _pos = self._io.pos()
        self._io.seek(0)
        self._m_header = Wfm1000b.Header(self._io, self, self._root)
        self._io.seek(_pos)
        return getattr(self, '_m_header', None)


